%% SIMULATION ANALYSIS
% Parameter recovery to check the robustness of the parameter estimation
% Note: Only for Lattimore PWF!

clear all

DataPath = 'C:\Users\kojala\Dropbox\Projects\CE_task_analyses\MATLAB_Data';
load(fullfile(DataPath,'gainsdata_alphaestimated_LattimorePWF_iter200_new'));
load(fullfile(DataPath,'lossesdata_alphaestimated_LattimorePWF_iter200_new'));

% Remove subjects that have been excluded from group analyses (18, 29, 30, 40, 48, 52)
excluded = [18 29 30 40 48 52];
resultdata_gains(ismember(resultdata_gains(:,1),excluded)==1,:) = [];
resultdata_losses(ismember(resultdata_losses(:,1),excluded)==1,:) = [];

% Initialize matrix for results
rows = length(resultdata_gains);
simdata_gains = zeros(rows, 21);
simdata_losses = zeros(rows, 21);

% Copy info columns (sub no., group, drug)
simdata_gains(:,1:3) = resultdata_gains(:,1:3);
simdata_losses(:,1:3) = resultdata_losses(:,1:3);

% Bounds for w(2/6) initial parameter (if no_iter > 1)
paraminit_bounds_w26 = [0 1];

% Bounds for alpha initial parameter (if no_iter > 1)
paraminit_bounds_a = [0 5];

% Bounds for PWF delta and gamma initial parameters (if no_iter > 1)
paraminit_bounds = [0 5];

% Gains/losses
for gainloss=1:2 % Gains = 1, Losses = 2
    
    if gainloss==1; gains=true; elseif gainloss==2; gains=false; end;
    
    for iter=1:200 % Repeat parameter simulation 200 times
        
        if gains==true;
            resultdata = resultdata_gains;
            simdata = simdata_gains;
            order26 = [1200 0; 600 0; 1200 600; 600 300; 1000 400; 1200 900];
            order = [1200 0; order26; 1200 0; 1200 0; 1200 0];
        elseif gains==false;
            resultdata = resultdata_losses;
            simdata = simdata_losses;
            order26 = [-1200 0; -600 0; -1200 -600; -600 -300; -1000 -400; -1200 -900];
            order = [-1200 0; order26; -1200 0; -1200 0; -1200 0];
        end;
        
        % Step 1: Generate certainty equivalents based on
        % individual PWF parameter estimates (alpha, delta, gamma)
        CEsim = simulate_noisydata(resultdata,rows,order,gains);
        simdata(:,4:13) = CEsim;
        
        for rowind=1:rows

            % Step 2: Retrieve individual PWF parameter estimates
            % using the generated CEs
            
            % Initial parameters w(2/6) and alpha
            % A random number for w(2/6) starting point value
            r_w26 = (paraminit_bounds_w26(2)-paraminit_bounds_w26(1)).*rand(1)+paraminit_bounds_w26(1);
            % A random number for alpha starting point value
            r_alpha = (paraminit_bounds_a(2)-paraminit_bounds_a(1)).*rand(1)+paraminit_bounds_a(1);
            parameters_initial = [r_w26 r_alpha];
            
            % Estimate w(2/6) and alpha parameter
            ydata = simdata(rowind,5:10);
            parameters_final = estimateCPT(parameters_initial,ydata,order26,true);
            simdata(rowind,[16 14]) = parameters_final;
            % Alpha to 14, w(2/6) to 16
            
            % Calculate subjective probability weights (Wp)
            % Except 2/6 which was estimated
            alpha = parameters_final(2);
            CEind = [4 11:13];
            Wpind = [15 17:19];
            
            Wp = calculateSPW(CEind,gains,simdata,rowind,alpha);
            simdata(rowind,Wpind) = Wp;
            
            % Compute the delta and gamma parameters of PWF
            % And fitted data points
            
            % Initial parameters
            % delta: elevation parameter
            % gamma: degree of distortion/curvature
            % A random number for w(2/6) starting point value
            r_delta = (paraminit_bounds(2)-paraminit_bounds(1)).*rand(1)+paraminit_bounds(1);
            % A random number for alpha starting point value
            r_gamma = (paraminit_bounds(2)-paraminit_bounds(1)).*rand(1)+paraminit_bounds(1);
            parameters_initial2 = [r_delta r_gamma];
            
            % Objective probability vector
            probsx = [0 1/6 2/6 3/6 4/6 5/6 1];
            
            % Indices for w(p)s
            Wp2ind = 15:19;
            
            parameters_final2 = computePWF(true,simdata,rowind,Wp2ind,parameters_initial2,probsx);
            simdata(rowind,20:21) = parameters_final2;
            
            simw26(rowind,iter) = parameters_final(1);
            simalpha(rowind,iter) = parameters_final(2);
            simdelta(rowind,iter) = parameters_final2(1);
            simgamma(rowind,iter) = parameters_final2(2);
            
        end
        
    end
    
    % Step 3: Save simulation data
    if gains==true %simdata_gains = simdata;
        save(fullfile(DataPath,'simulationparams_gains_withnoise_iter200'),'simw26','simalpha','simdelta','simgamma')
    elseif gains==false %simdata_losses = simdata;
        save(fullfile(DataPath,'simulationparams_losses_withnoise_iter200'),'simw26','simalpha','simdelta','simgamma')
    end

end

% Step 4: Compare original and simulated parameter estimates
placeboind = find(resultdata_gains(:,3)==1); % same for gains and losses
sulpirideind = find(resultdata_gains(:,3)==2);

% Get the MEDIAN and standard deviation of the simulated data over all
% iterations for each subject, for all conditions separately
[simmean, simstd] = calculateSumStatsSimOrigData(DataPath,placeboind,sulpirideind);

paramind = [14 16 20 21];

% Calculate the median over all participants of the individual medians of the
% simulated data
for gainloss = 1:2
    
    if gainloss == 1
        resultdata = resultdata_gains;
    else
        resultdata = resultdata_losses;
    end
    
    for param = 1:length(paramind)
        
        placebo = resultdata(placeboind,paramind(param));
        sulpiride = resultdata(sulpirideind,paramind(param));
        
        diffP(gainloss,param,:) = placebo-squeeze(simmean(gainloss,param,1,:));
        diffS(gainloss,param,:) = sulpiride-squeeze(simmean(gainloss,param,2,:));
        
        simmedP = squeeze(simmean(gainloss,param,1,:));
        simmedS = squeeze(simmean(gainloss,param,2,:));
        
        medianP(gainloss,param) = median(simmedP);
        medianS(gainloss,param) = median(simmedS);
        
        [rhoP(gainloss,param)] = corr(simmedP,placebo,'type','Spearman'); % Correlation original vs. simulated
        [rhoS(gainloss,param)] = corr(simmedS,sulpiride,'type','Spearman');
        
        if gainloss == 1 && param == 4 % Gamma gains
            [p,h,stats] = signrank(simmedS,simmedP); % Test difference between sulpiride and placebo (vs. original main result test)
        end
        
    end
    
end

save(fullfile(DataPath,'simulationparams_withnoise_correlations_iter200_bothCE'),'rhoP','rhoS')

% Plotting the differences for all parameters and conditions
param = 1:4;
parname = {'Alpha' 'w(2/6)' 'Delta' 'Gamma'};

for p = param

    figure
    suptitle(parname{p})
    hold on
    
    subplot(2,2,1)
    bar(squeeze(diffP(1,p,:)))
    title('Gains Placebo')
    
    subplot(2,2,2)
    bar(squeeze(diffS(1,p,:)))
    title('Gains Sulpiride')
    
    subplot(2,2,3)
    bar(squeeze(diffP(2,p,:)))
    title('Losses Placebo')
    
    subplot(2,2,4)
    bar(squeeze(diffS(2,p,:)))
    title('Losses Sulpiride')
    
end

% For each parameter, plot original vs. simulated parameters
paramCorrPlots(resultdata,simdata,gains)