%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Certainty equivalent task (see Abdellaoui et al. 2011 for reference)                                                                %%%                                                                                                                          %%%
%%% Karita Ojala - Donders Institute - 08/07/2016                                                                                       %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Last edited: 15/08/2017

clear all;

% Drug or Session effect
drug_effect       = true;  % true = sessions sorted according to drug received; false = sessions sorted according to session order
if drug_effect % if sorting sessions based on drug received
    session_names = {'Placebo'; 'Sulpiride'};
else
    session_names = {'Day1'; 'Day2'};
end

% Parameters
nb_session  = 2; % number of sessions/test days
nb_run      = 2; % number of runs
groups      = {'controls' 'gamblers'};
subjects    = {[(1:20) 22 23];... % CONTROLS % Exp21 does not exist in the data
               [(29:37) (39:42) (44:50) 52]}; % GAMBLERS % Exp38 excluded as only has session 1 % Exp43,51 does not exist in the data
               % NOTE: if exclusion -> do NOT edit 'drug_order', only edit 'subjects' 
% Drug order for ALL participants - 1 = placebo; 2 = sulpiride (as provided by the pharmacy 14-05-2013)
drug_order  = [1 2;1 2;2 1;2 1;  2 1;1 2;1 2;2 1;  2 1;2 1;1 2;1 2;  2 1;1 2;2 1;1 2;  2 1;1 2;2 1;1 2;  1 2;2 1;2 1;1 2;  2 1;1 2;1 2;2 1;... % CONTROLS
               2 1;1 2;2 1;1 2;  1 2;2 1;1 2;2 1;  1 2;2 1;2 1;1 2;  1 2;2 1;2 1;1 2;  2 1;1 2;2 1;1 2;  2 1;1 2;1 2;2 1;  1 2;2 1;1 2;2 1]; % GAMBLERS

%% START CODE

%% OPTIONS FOR THE ESTIMATION PROCEDURE

% -- General options --
mainpath = 'C:\Users\kojala\Dropbox\Projects\CE_task_analyses\';

% Data folder

datafolder = [mainpath 'DATA\']; 

% Saving folder

savefolder = [mainpath 'MATLAB_Data\'];

% Overwrite/save data
datasaved = false;

% Number of iterations (how many times the whole loop over subjects is run)

% The iteration with the smallest residual value is chosen to remain as the
% final estimation result

% Choose 1 if no iterations, use iterations to avoid local minima/maxima in the estimation
% Also gives no. of iterations for delta & gamma estimation
% 40 iterations should be enough according to Payam
% To ensure not getting stuck in local maxima
% But seems better to be sure and have more

no_iter = 200;
        
% -- Value function options --

% Estimation of alpha
%   Is alpha estimated: true. If not (alpha = 1): false.
%   If alpha is not estimated -> linear value function
alphaestimated = true;

% Bounds for w(2/6) initial parameter (if no_iter > 1)
paraminit_bounds_w26 = [0 1];

% Bounds for alpha initial parameter (if no_iter > 1)
paraminit_bounds_a = [0 5];

% Initial value for alpha and w(2/6) (if no_iter = 1)
% E.g., based on literature (here from Fox & Poldrack 2008)
alphainit = 0.88;
w26init = 0.33;

% -- Probability weighting function options --

% Type of PWF
%   Lattimore log-odds metric: true
%   Prelec 2-parameter: false
pwf = true;

% Bounds for PWF delta and gamma initial parameters (if no_iter > 1)
paraminit_bounds = [0 5];

% Initial value for alpha and w(2/6) (if no_iter = 1)
% E.g., based on literature (here from Fox & Poldrack 2008)
deltainit = 0.8;
gammainit = 0.6;

% Data matrix indices
% 1: Subject number
% 2: Group
% 3: Drug
% 4-13: Certainty equivalents for the 10 gambles
% 14: Alpha
% 15-19: Probability weights for the 5 different gamble probabilities
% 20: Delta
% 21: Gamma
% 22-26: Normalized certainty equivalents
% 27: Smallest resnorm from alpha and w(2/6) estimation repetitions
% 28: Smallest resnorm from delta and gamma estimation repetitions

% !! Go to the end of the code to change the name of the saved file !!
                
%% STEP 1: Individual certainty equivalent and prospect theory parameter estimation

% Initialize matrix for results
% Rows: subjects(43)*sessions(2) = 86
% Columns: variables = 40
resultdata_gains = zeros(2*(length(subjects{1})+length(subjects{2})), 40);
resultdata_losses = zeros(2*(length(subjects{1})+length(subjects{2})), 40);

% Index for later matrix allocation (2 rows per subject as 2 sessions)
mat_index = 1;

% Initialized variables for saving the number of changes in resnorm values
% (sensitivity analysis for the stability of the estimation procedure)
resnormchange_awg = 0; % Alpha & w(2/6) gains
resnormchange_awl = 0; % Alpha & w(2/6) losses
resnormchange_dgg = 0; % Delta & gamma gains
resnormchange_dgl = 0; % Delta & gamma losses

%--

for group_id=1:length(groups) % LOOP ON GROUPS: Controls & Gamblers
    sub_list=subjects{group_id}; % get subject numbers
  
    for subject_id=sub_list % LOOP ON SUBJECTS
        disp(subject_id); % Show subject ID when the code is running

        for session_id=1:nb_session % LOOP ON SESSIONS (i.e. testdays)

            drug_id = drug_order(subject_id,session_id); % 1 = placebo; 2 = sulpiride
            if drug_effect % if sorting sessions based on drug received
                session_index = drug_id; % session_index will be used to group/average similar sessions together for graphs and logistic regression
            else           % if sorting sessions based on test day
                session_index = session_id; %#ok<*UNRCH> % Suppress yellow "error"
            end
            disp(session_names{session_index});

            % Data initialization
            rawdata = dataInit(datafolder,subject_id,session_id);
            
            % All tested blocks in the order they were presented in
            blocks = unique(rawdata(:,2),'stable');
                
            % Check that both blocks have been tested
            if numel(blocks)<2; error(['At least one missing block in subject: ' int2str(subject_id)]);end;
            
            for run_id=1:nb_run % LOOP ON RUNS/BLOCKS (i.e. losses & gains)
                
                % Find the trials with 'verified' subjective equivalence points
                % (when a participant has passed the consistency check)
                equivalence_indices(:,1) = find(rawdata(:,11)==1);
                equivalence_indices(:,2) = rawdata(equivalence_indices(:,1),3);
                
                % Probabilities associated with these verified subjective equivalence
                % points, in ascending order
                probabilities = sort(equivalence_indices(1:10,2));
                % Note: includes 6x 2/6 probabilities!

                % Check that all probabilities have been tested (1/6, 6x 2/6, 3/6, 4/6, 5/6)
                if numel(probabilities)<10; error(['At least one missing probability in subject: ' int2str(subject_id)]);end
                
                % Check that all probabilities have an equivalence point
                if numel(unique(rawdata(equivalence_indices(:,1),3)))~=5; error('Number of certainty equivalence points and probabilities not the same!');end
             
                % Determine whether the block is 1 (gains) or 2 (losses)
                if blocks(run_id)==1; gains=true; elseif blocks(run_id)==2; gains=false;end
                
                % Initialize matrix for monetary value information
                xdata = zeros(6,2);
                % Growing index for probabilities 2/6
                ind_p33 = 0;
                
                for p = 1:length(probabilities) % Loop over probabilities

                    % Get indices for certainty equivalents (CEs) and associated probabilities
                    [indice,indprob] = getCEIndex(run_id,equivalence_indices,probabilities(p),ind_p33);
                    % Compute certainty equivalents
                    CE = computeCE(rawdata,indice);
                    
                    % Checking if probability is 2/6, a growing index for
                    % it and creating xdata (the money values in order of
                    % presentation)
                    if (indprob==probabilities(2))
                        ind_p33 = ind_p33 + 1;
                        xdata(ind_p33,1) = rawdata(indice,4);
                        xdata(ind_p33,2) = rawdata(indice,5);
                    end
  
                    % Assign results in the correct matrix (gains or losses)
                    if gains==true;
                        resultdata_gains(mat_index,1) = subject_id;
                        resultdata_gains(mat_index,2) = group_id;
                        resultdata_gains(mat_index,3) = drug_id;
                        resultdata_gains(mat_index,(3+p)) = CE;
                    elseif gains==false;
                        resultdata_losses(mat_index,1) = subject_id;
                        resultdata_losses(mat_index,2) = group_id;
                        resultdata_losses(mat_index,3) = drug_id;
                        resultdata_losses(mat_index,(3+p)) = CE;
                    end
                end
                
                % REORDER CEs (in case p = 2/6)
                if gains==true;
                    targetorder_gains = [1200 0; 600 0; 1200 600; 600 300; 1000 400; 1200 900];
                    CEr = reorderCE(xdata,targetorder_gains,resultdata_gains,mat_index);
                    resultdata_gains(mat_index,5:10) = CEr;
                elseif gains==false;
                    targetorder_losses = [-1200 0; -600 0; -1200 -600; -600 -300; -1000 -400; -1200 -900];
                    CEr = reorderCE(xdata,targetorder_losses,resultdata_losses,mat_index);
                    resultdata_losses(mat_index,5:10) = CEr;
                end
                
                % ESTIMATE w(2/6) AND VALUE FUNCTION CURVATURE PARAMETER
                
                for i=1:no_iter % Loop over iterations
                    
                    % Choosing the starting values for the parameters in
                    % the estimation
                    if no_iter > 1 % If iterations
                        % A random number for w(2/6) starting point value
                        r_w26 = (paraminit_bounds_w26(2)-paraminit_bounds_w26(1)).*rand(1)+paraminit_bounds_w26(1);
                        % A random number for alpha starting point value
                        r_alpha = (paraminit_bounds_a(2)-paraminit_bounds_a(1)).*rand(1)+paraminit_bounds_a(1);
                        parameters_initial = [r_w26 r_alpha];
                    elseif no_iter == 1 % In case of no iterations
                        % From literature: Fox & Poldrack 2008
                        parameters_initial = [w26init alphainit];
                    end
                
                    % Estimated parameters, w(2/6) and alpha
                    if gains==true; % Gains

                        clear ydata parameters_final resnorm
                        ydata = resultdata_gains(mat_index,5:10); % Retrieval of CE(2/6) values
                        % Estimation step of the value function and w(2/6)
                        % CPT = Cumulative Prospect Theory
                        % Input: 
                        %   - Starting values for the estimated parameters
                        %   - Certainty equivalent data for p = 2/6
                        %   - The order of gambles, see Abdellaoui et al.
                        %   - Boolean on whether alpha is estimated or not
                        [parameters_final, resnorm, residual] = estimateCPT(parameters_initial,ydata,targetorder_gains,alphaestimated);
                        % Output: estimated values of alpha and w(2/6)
                                                
                        paramsfinal_gains{mat_index,1}(1:2,i) = parameters_final; %#ok<SAGROW>
                        resnorm_gains{mat_index,1}(1,i) = resnorm; %#ok<SAGROW>
                        
                        allresnorm_aw26_gains(i) = resnorm;
                        allresidual_aw26_gains(i,:) = residual;
                        
                        if resultdata_gains(mat_index,27)==0 || resnorm <= resultdata_gains(mat_index,39)
                            if resnorm <= resultdata_gains(mat_index,39); resnormchange_awg = resnormchange_awg+1; end
                            resultdata_gains(mat_index,27) = resnorm;
                            resultdata_gains(mat_index,[16 14]) = parameters_final; % Allocation of results in the final result matrix
                            % Alpha in 14, w(2/6) in 16, later w(1/6) in 15
                        end
                        
                    elseif gains==false; % Losses

                        clear ydata parameters_final resnorm
                        ydata = resultdata_losses(mat_index,5:10);
                        [parameters_final, resnorm, residual] = estimateCPT(parameters_initial,ydata,targetorder_losses,alphaestimated);

                        paramsfinal_losses{mat_index,1}(1:2,i) = parameters_final; %#ok<SAGROW>
                        resnorm_losses{mat_index,1}(1,i) = resnorm; %#ok<SAGROW>
                        
                        allresnorm_aw26_losses(i) = resnorm;
                        allresidual_aw26_losses(i,:) = residual;
                        
                        if resultdata_losses(mat_index,27)==0 || resnorm <= resultdata_losses(mat_index,39)
                            if resnorm <= resultdata_losses(mat_index,39); resnormchange_awl = resnormchange_awl+1; end
                            resultdata_losses(mat_index,27) = resnorm;
                            resultdata_losses(mat_index,[16 14]) = parameters_final;
                        end
                        
                    end
                
                end
                
                if gains==true;
                    %save(fullfile(savefolder,'Resnorm',['resnormdata_aw26_gains_iter' num2str(no_iter) '_subject' num2str(subject_id) '_drug' num2str(drug_id) '.mat']),'allresnorm_aw26_gains')
                    save(fullfile(savefolder,'Residual',['residualdata_aw26_gains_iter' num2str(no_iter) '_subject' num2str(subject_id) '_drug' num2str(drug_id) '.mat']),'allresidual_aw26_gains')
                elseif gains==false;
                    %save(fullfile(savefolder,'Resnorm',['resnormdata_aw26_losses_iter' num2str(no_iter) '_subject' num2str(subject_id) '_drug' num2str(drug_id) '.mat']),'allresnorm_aw26_losses')
                    save(fullfile(savefolder,'Residual',['residualdata_aw26_losses_iter' num2str(no_iter) '_subject' num2str(subject_id) '_drug' num2str(drug_id) '.mat']),'allresidual_aw26_losses')
                end
                
                % CALCULATE (SUBJECTIVE) PROBABILITY WEIGHTS &
                % NORMALIZED CERTAINTY EQUIVALENTS

                % alpha = value function curvature, gotten from previous
                % estimation function
                if alphaestimated == true
                    alpha = parameters_final(2);
                else % Alpha not estimated, = 1, linear value function
                    alpha = 1; % Linear value function
                    resultdata_gains(mat_index,14) = alpha;
                    resultdata_losses(mat_index,14) = alpha;
                end
                
                % Vector with indexes of the wanted CE columns and right
                % columns for the results to be placed in
                CEind = [4 11:13];
                nCEind = [4 5 11:13]; % Including 2/6(1)
                resplace = [15 17:19];
                nCEresplace = 22:26;
                
                % Calculate subjective probability weights
                % and normalized certainty equivalents
                if gains==true; % Gains
                    
                    resultdata_gains(mat_index,resplace) = calculateSPW(CEind,gains,resultdata_gains,mat_index,alpha);
                    resultdata_gains(mat_index,nCEresplace) = calculateNormCE(nCEind,gains,resultdata_gains,mat_index,alpha);
                    
                elseif gains==false; % Losses
                    
                    resultdata_losses(mat_index,resplace) = calculateSPW(CEind,gains,resultdata_losses,mat_index,alpha);
                    resultdata_losses(mat_index,nCEresplace) = calculateNormCE(nCEind,gains,resultdata_losses,mat_index,alpha);
                    
                end

                % DERIVE PROBABILITY WEIGHTING FUNCTION (PWF) PARAMETERS
                % (SPECIFIC SHAPE OF THE FUNCTION)

                for i=1:no_iter
                    
                    % Initial parameters
                    % delta: elevation parameter
                    % gamma: degree of distortion/curvature
                    
                    % Choosing the starting values for the parameters in
                    % the estimation
                    if no_iter > 1
                        % A random number for w(2/6) starting point value
                        r_delta = (paraminit_bounds(2)-paraminit_bounds(1)).*rand(1)+paraminit_bounds(1);
                        % A random number for alpha starting point value
                        r_gamma = (paraminit_bounds(2)-paraminit_bounds(1)).*rand(1)+paraminit_bounds(1);
                        parameters_initial2 = [r_delta r_gamma];
                    elseif no_iter == 1
                        % Values from the literature: Fox & Poldrack, 2008
                        parameters_initial2 = [deltainit gammainit];
                    end

                    % Objective probability vector
                    %probsx = [1/6 2/6 3/6 4/6 5/6];
                    probsx = [0 1/6 2/6 3/6 4/6 5/6 1];
                    % For fitted points with higher resolution
                    probsxfit = 0:0.01:1;
                    %probsxfit = probsxfit(2:end-1);
                    
                    % Source indexes of w(p)s
                    Wpind = 15:19;
                    
                    % Main fitting step, estimating delta and gamma
                    if gains==true; % Gains
                        
                        % Estimation function
                        % Input: 
                        %   - Type of PWF
                        %   - Result data matrix
                        %   - Current row index in the matrix
                        %   - Source indexes for w(p)s
                        %   - Starting parameters for delta and gamma
                        %   - Objective probability vector
                        [parameters_final2, resnorm, residual] = computePWF(pwf,resultdata_gains,mat_index,Wpind,parameters_initial2,probsx);
                        % Output: Estimated parameters for delta and gamma
                        allresnorm_dg_gains(i) = resnorm;
                        allresidual_dg_gains(i,:) = residual;
                        
                        % Step for finding the smallest resnorm value and
                        % saving the parameter values associated with that
                        % to avoid getting stuck in local maxima/minima
                        if resultdata_gains(mat_index,28)==0 || resnorm <= resultdata_gains(mat_index,40)
                            if resnorm <= resultdata_gains(mat_index,40); resnormchange_dgg = resnormchange_dgg+1; end
                            resultdata_gains(mat_index,28) = resnorm;
                            resultdata_gains(mat_index,20:21) = parameters_final2;
                            %resultdata_gainsfit(mat_index,:) = computeFittedDatap(pwf,parameters_final2,probsxfit); %#ok<SAGROW>
                        end
                        
                    elseif gains==false; % Losses
                        
                        [parameters_final2, resnorm, residual] = computePWF(pwf,resultdata_losses,mat_index,Wpind,parameters_initial2,probsx);
                        allresnorm_dg_losses(i) = resnorm;
                        allresidual_dg_losses(i,:) = residual;
                        
                        if resultdata_losses(mat_index,28)==0 || resnorm <= resultdata_losses(mat_index,40)
                            if resnorm <= resultdata_losses(mat_index,40); resnormchange_dgl = resnormchange_dgl+1; end
                            resultdata_losses(mat_index,28) = resnorm;
                            resultdata_losses(mat_index,20:21) = parameters_final2;
                            %resultdata_lossesfit(mat_index,:) = computeFittedDatap(pwf,parameters_final2,probsxfit); %#ok<SAGROW>
                        end
                        
                    end
                    
                end
                
                if gains==true;
                    %save(fullfile(savefolder,'Resnorm',['resnormdata_dg_gains_iter' num2str(no_iter) '_subject' num2str(subject_id) '_drug' num2str(drug_id) '.mat']),'allresnorm_dg_gains')
                    save(fullfile(savefolder,'Residual',['residualdata_dg_gains_iter' num2str(no_iter) '_subject' num2str(subject_id) '_drug' num2str(drug_id) '.mat']),'allresidual_dg_gains')
                elseif gains==false;
                    %save(fullfile(savefolder,'Resnorm',['resnormdata_dg_losses_iter' num2str(no_iter) '_subject' num2str(subject_id) '_drug' num2str(drug_id) '.mat']),'allresnorm_dg_losses')
                    save(fullfile(savefolder,'Residual',['residualdata_dg_losses_iter' num2str(no_iter) '_subject' num2str(subject_id) '_drug' num2str(drug_id) '.mat']),'allresidual_dg_losses')
                end
                    
                 % COMPUTE PWF PARAMETERS AND FITTED DATAPOINTS FOR NORMALIZED CEs
%                  Wpind = nCEresplace; % Indices for the normalized CEs
%                  
%                  if gains==true; % Gains
%                      
%                      % Compute PWF parameters with normalized CEs
%                      parameters_final_nCE = computePWF(pwf,resultdata_gains,mat_index,Wpind,parameters_initial2,probsx);
%                      resultdata_gains(mat_index,32:33) = parameters_final_nCE; % Save in the result data matrix
%                      
%                      % Compute fitted data points for normalized CEs
%                      resultdata_gains(mat_index,34:38) = computeFittedDatap(pwf,parameters_final_nCE,probsx);
%                      
%                  elseif gains==false; % Losses
%                      
%                      parameters_final_nCE = computePWF(pwf,resultdata_losses,mat_index,Wpind,parameters_initial2,probsx);
%                      resultdata_losses(mat_index,32:33) = parameters_final_nCE;
%                      
%                      resultdata_losses(mat_index,34:38) = computeFittedDatap(pwf,parameters_final_nCE,probsx);
% 
%                  end
                 
            end
            mat_index = mat_index+1; % Increasing index to go to next row
        end     
    end
end

% Saving result data matrices in .mat files
% Remember to change name to avoid overwriting!

% Generate name based on options
if alphaestimated == false
    filename = 'alphalinear';
else
    filename = 'alphaestimated';
end

if pwf == true
    filename = [filename '_LattimorePWF_iter' num2str(no_iter)];
else
    filename = [filename '_Prelec2PWF_iter' num2str(no_iter)];
end

if datasaved == true
    %save([savefolder 'gainsdata_' filename],'resultdata_gains')
    save([savefolder 'gainsdata_' filename '_version'],'resultdata_gains')
    %save([savefolder 'lossesdata_' filename],'resultdata_losses')
    save([savefolder 'lossesdata_' filename '_version'],'resultdata_losses')
end

%% STEP 2: Group analyses    
% Performed in SPSS and described in the article
% SPSS data files and syntax available